package support.redhat.entity.relationship;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.EntityTransaction;
import javax.persistence.Persistence;

import support.redhat.util.AutoCloser;
import junit.framework.TestCase;

import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Fetch;
import javax.persistence.criteria.Join;
import javax.persistence.criteria.JoinType;
import javax.persistence.criteria.Root;

import test.Entity1;
import test.Entity2;
import test.Entity3;
import test.Entity1_;
import test.Entity2_;
import test.Entity3_;

public class TestHibernate extends TestCase {
	private EntityManager entityManager;

	private final AutoCloser m_closer = new AutoCloser();

	@Override
	protected void setUp() throws Exception {
		try {
			// Configures settings from annotations + persistence.xml
			EntityManagerFactory emf = Persistence.createEntityManagerFactory("TEST");
			m_closer.push(emf);
			entityManager = emf.createEntityManager();
			m_closer.push(entityManager);
		} catch (Throwable t) {
			t.printStackTrace();
		}
	}

	@Override
	protected void tearDown() throws Exception {
		if (entityManager != null) {
			m_closer.close();
			entityManager = null;
		}
	}

	public void test() {
		try {
			final CriteriaBuilder builder = entityManager.getCriteriaBuilder();
			final CriteriaQuery<Entity1> criteria = builder.createQuery(Entity1.class);

			final Root<Entity1> root = criteria.from(Entity1.class);
			final Join<Entity1, Entity2> entity2Join = root.join(Entity1_.entity2, JoinType.INNER); // illegal with fetch join

			final Fetch<Entity1, Entity2> entity2Fetch = root.fetch(Entity1_.entity2, JoinType.INNER); // <=== REMOVE
			entity2Fetch.fetch(Entity2_.entity3); // <=== REMOVE

			criteria.where(builder.equal(root.get(Entity1_.value), "test"),
					builder.equal(entity2Join.get(Entity2_.value), "test")); // illegal with fetch join

			entityManager.createQuery(criteria).getResultList();
		} catch (Throwable t) {
			t.printStackTrace();
			fail(t.getMessage());
		}
	}
}
